<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Repositories;

use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Prettus\Repository\Eloquent\BaseRepository as BaseRepositoryEloquent;

abstract class BaseRepository extends BaseRepositoryEloquent
{
    public function listPageByPaginate($limit = null, $columns = ['*'], $paginateMethod = 'paginate'): array
    {
        $paginateResult = $this->paginate($limit,$columns,$paginateMethod);

        $paginate = [];
        $paginate['data'] = $paginateResult->items();
        $paginate['current_page'] = $paginateResult->currentPage();
        $paginate['total'] = $paginateResult->total();

        return $paginate;
    }

    public function transformData($datas, $modelName = null)
    {
        if(is_null($modelName)){
            return Arr::pluck($datas, null);
        }
        return app($modelName)->transform(Arr::pluck($datas, null));
    }

    public function getConnectionName()
    {
        return $this->makeModel()->getConnectionName();
    }

    public function getTableName()
    {
        return $this->makeModel()->getTable();
    }

    public function getConnection()
    {
        return DB::connection($this->getConnectionName());
    }
}
