<?php

namespace App\Services\ThirdPlatform;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Rhawn\Services\RhawnProductService;
use App\Services\ThirdPlatform\Api\WuxiApiNewService;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class WuxiLabNewService
{

    public function __construct()
    {
        $this->apiService = (new WuxiApiNewService(app(ThirdApiPlatformRepository::class)));
        $this->rhawnChemicalsService = app(RhawnProductService::class);
    }

    public function initToken()
    {
        return $this->getToken();
    }

    /**
     * 批量更新商品
     */
    public function batchUpdateProducts($type = 'all')
    {
        $status = $this->apiService->checkPlatformStatus();
        if($status){
            $limit = $this->apiService->getPlatformInfo('platform_params')['batchNums'];
            $page = 0;

            ini_set('memory_limit','3072M');
            while(true) {
                $rawList = $this->rhawnChemicalsService->getRawsList($page * $limit, $limit);
                if (!$rawList) {
                    break;
                }
                $rawIdList = [];
                foreach ($rawList as $raw) {
                    $rawIdList[] = $raw['r_id'];
                }
                $packages = $this->rhawnChemicalsService->getChemicalPackage($rawIdList);
                $newPackages = [];
                if($packages){
                    foreach($packages as $pack){
                        //符合110状态才上架
                        if($pack['p_status'] == 1 && $pack['p_show'] == 1 && $pack['p_if_big'] == 0){
                            $newPackages[$pack['p_code']] = $pack;
                        }
                        //$newPackages[$pack['r_code']] = $pack;
                    }
                }

                $queueParams = [];
                switch($type){
                    case 'all': $queueParams = [
                        'params' => ['rawList' => $rawList,'packageList' => $newPackages],
                        'consumer' => __CLASS__,
                        'method' => 'queueBatchUpdateProductToApi'
                    ];break;
                    case 'package': $queueParams = [
                        'params' => ['packageList' => $newPackages],
                        'consumer' => __CLASS__,
                        'method' => 'queueBatchUpdatePackagesToApi'
                    ];break;
                    default:
                        $queueParams = [
                            'params' => ['rawList' => $rawList,'packageList' => $newPackages],
                            'consumer' => __CLASS__,
                            'method' => 'queueBatchUpdateProductToApi'
                        ];
                }

                if(!empty($rawList)){
                    $this->apiService->pushQueue($queueParams,'wuxilab_api');
                }
                $page ++;
            }
        }
    }

    /**
     * 批量更新产品价格
     * @return void
     * @throws \Exception
     */
    public function batchUpdateProductPrice()
    {
        $status = $this->apiService->checkPlatformStatus();
        if($status){
            $dbConnect = DB::connection('rhawn_mysql');

            $startime = date('Y-m-d', strtotime('-1 day')).' 00:00:00';
            $endtime = date('Y-m-d', time()).' 00:00:00';

            $product = $dbConnect->table('prochange')
                ->whereRaw("updatetime >=".strtotime($startime)." and updatetime <=".strtotime($endtime))
                ->where('type', '2')
                ->get()->toArray();
            if(count($product) > 0){
                $productIds = array_unique(array_column($product, 'p_id'));
                $productList = $this->rhawnChemicalsService->getProductsListThroughPids($productIds);

                $updateProductPrice = [];
                foreach($productList as $product){
                    $tempData = [];
                    $tempData['product_division'] = 'CHEMICAL';
                    list($pCode, $packCode) = explode('-', $product->p_code);
                    $tempData['productId'] = $pCode;
                    $tempData['packageUnitDescription'] = $product->p_code;
                    $tempData['minPackageUnitDescription'] = '';
                    $tempData['price'] = bcmul($product->p_price, 0.5, 2);
                    $tempData['discount'] = '0.5';
                    $tempData['inCostCurrency'] = 'RMB';
                    $tempData['status'] = $product->p_status == '1' ? 'ACTIVE' : 'INACTIVE';;
                    $updateProductPrice[] = $tempData;
                }
                $result = $this->apiService->pushBatchUpdatePrice($updateProductPrice, $this->getToken());
                $this->processApiResponse($result);
            }

        }
    }

    public function batchUpdateProductStock()
    {
        $status = $this->apiService->checkPlatformStatus();
        if($status){
            $dbConnect = DB::connection('rhawn_mysql');

            $startime = date('Y-m-d', strtotime('-1 day')).' 00:00:00';
            $endtime = date('Y-m-d', time()).' 00:00:00';

            $product = $dbConnect->table('prochange')
                ->whereRaw("updatetime >=".strtotime($startime)." and updatetime <=".strtotime($endtime))
                ->where('type', '1')
                ->get()->toArray();
            if(count($product) > 0){
                $productIds = array_unique(array_column($product, 'p_id'));
                $productList = $this->rhawnChemicalsService->getProductsListThroughPids($productIds);

                $updateProductStock = [];
                foreach($productList as $product){
                    $tempData = [];
                    $tempData['product_division'] = 'CHEMICAL';
                    list($pCode, $packCode) = explode('-', $product->p_code);
                    $tempData['productId'] = 'R003250';//$pCode;
                    $tempData['packageUnitDescription'] = 'R003250-1ml';//$product->p_code;
                    $tempData['packageStockQuantity'] = $product->p_stock;
                    $tempData['packageStockLocation'] = 'CN-SH';
                    $tempData['stockStatus'] = $product->p_status == '1' ? 'ACTIVE' : 'INACTIVE';;
                    $updateProductStock[] = $tempData;
                    break;
                }
                $result = $this->apiService->pushBatchUpdateStock($updateProductStock, $this->getToken());
                $this->processApiResponse($result);
            }

        }
    }

    public function queueBatchUpdatePackagesToApi($updateData)
    {
        if(empty($updateData['packageList'])){
            return false;
        }
        try{
            $packageList = [];
            foreach($updateData['packageList'] as $packageCode => $data){
                $tempData = [];
                $tempData['product_division'] = 'CHEMICAL';
                $tempData['productId'] = $data['r_code'];
                $tempData['status'] = $data['p_status'] == '1' ? 'ACTIVE' : 'INACTIVE';
                $tempData['packageUnitDescription'] = $packageCode;
                $packageList[] = $tempData;
            }

            $result = $this->apiService->pushBatchUpdatePackages($packageList, $this->getToken());
            $this->processApiResponse($result);

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
            var_dump($exception->getMessage());
        }
    }

    /**
     * @批量更新商品
     * @param $updateData
     */
    public function queueBatchUpdateProductToApi($updateData)
    {
        if(empty($updateData['rawList']) || empty($updateData['packageList'])){
            return false;
        }
        try{
            $noSmiles = [];
            foreach($updateData['rawList'] as $data){
                if(empty($data['c_smiles']) && !empty($data['c_cas'])){
                    $noSmiles[] = $data['c_cas'];
                }
            }
            $updateData['rawList'] = array_column($updateData['rawList'], null, 'c_cas');

            if(!empty($noSmiles)){
                $noSmiles = array_unique($noSmiles);
                $baikeMysql = DB::connection('baike_mysql');
                $casResult = $baikeMysql->table('tp_mol_data')->whereIn('cas', $noSmiles)->get()->toArray();
                if($casResult){
                    foreach($casResult as $cas){
                        $updateData['rawList'][$cas->cas]['c_smiles'] = $cas->smiles;
                    }
                }
            }

            $result = $this->apiService->pushBatchUpdateProduct($updateData, $this->getToken());
            $this->processApiResponse($result);

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':batchUpdateToWuxiLab', 'error');
            var_dump($exception->getMessage());
        }
    }

    protected function processApiResponse($apiResponse)
    {
        if($apiResponse){
            //处理更新不成功的商品
            if(!empty($apiResponse['errorProduct'])){
                $errorProductsList = $this->apiService->getPlatformDataEntries('wuxiLab_error_products','data_values');
                foreach($apiResponse['errorProduct'] as $errorProduct){
                    if(!is_null($errorProductsList) && !empty($errorProductsList['products'])){
                        if(!in_array($errorProduct['data'],$errorProductsList['products'])){
                            array_push($errorProductsList['products'],$errorProduct['data']);
                        }
                    }else{
                        $errorProductsList['products'][] = $errorProduct['data'];
                    }
                }
                $this->apiService->storePlatformDataEntries('wuxiLab_error_products',['products' => $errorProductsList['products']]);
            }
            if(!empty($apiResponse['errorPackage'])) {
                $errorPackagesList = $this->apiService->getPlatformDataEntries('wuxiLab_error_packages', 'data_values');
                //更新成功的商品
                foreach ($apiResponse['errorPackage'] as $errorPackage) {
                    if (!is_null($errorPackagesList) && !empty($errorPackagesList['products'])) {
                        if (!in_array($errorPackage['data'], $errorPackagesList['products'])) {
                            array_push($errorPackagesList['products'], $errorPackage['data']);
                        }
                    } else {
                        $errorPackagesList['products'][] = $errorPackage['data'];
                    }
                }
                $this->apiService->storePlatformDataEntries('wuxiLab_error_packages', ['products' => $errorPackagesList['products']]);
            }
        }
    }

    /**
     * @ 获取api token
     * @return false|mixed
     */
    public function getToken(){
        try{
            $token = $this->apiService->getPlatformDataEntries('wuxilab_token','data_values');
            if(empty($token) || (time() > $token['ttl'])){
                $token = $this->apiService->getToken();
                if($token && $token['code'] == 200){
                    $this->apiService->storePlatformDataEntries('wuxilab_token',['token' => $token['data'], 'ttl' => strtotime('+2 hour')]);
                    return $token['data'];
                }
            }
            return $token['token'];
        }catch(\Exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':token error', 'error');
            return false;
        }
    }
}
